-------------------------------------------------------------------------------
-- File name : mips.erfile.vhdl
-- Title     : 32 x 32-bit register file
-- project   : ECE 485
-- Library   : single-cycle processor
-- Author(s) : James E. Stine, Jr.
-- Purpose   : definition of single-cycle processor
-- notes :   

-- Copyright Illinois Institute of Technology
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity mipssc is
  port (clk            : in  std_logic;
        reset          : in  std_logic;
        instr          : in  std_logic_vector (31 downto 0);
        data_read      : in  std_logic_vector (31 downto 0);
        ack            : in  std_logic;
        ins_addr       : out std_logic_vector (8 downto 0);
        data_addr      : out std_logic_vector (8 downto 0);
        mem_write_data : out std_logic_vector (31 downto 0);
        stb            : out std_logic;
        we             : out std_logic);
end mipssc;

architecture struct of mipssc is

  component mux21x5 
    port (A : in  std_logic_vector(4 downto 0);
          B : in  std_logic_vector(4 downto 0);
          S : in  std_logic;
          Y : out std_logic_vector(4 downto 0));
  end component;

  component mux21x9
    port (A : in  std_logic_vector(8 downto 0);
          B : in  std_logic_vector(8 downto 0);
          S : in  std_logic;
          Y : out std_logic_vector(8 downto 0));
  end component;

  component mux21x32 
    port (A : in  std_logic_vector(31 downto 0);
          B : in  std_logic_vector(31 downto 0);
          S : in  std_logic;
          Y : out std_logic_vector(31 downto 0));
  end component;

  component register9 
    port (d   :   in std_logic_vector(8 downto 0);
          clk :   in std_logic;
          reset : in std_logic;
          set :   in std_logic;
          q   :   out std_logic_vector(8 downto 0));
  end component;

  component rca9 
    port(A : in std_logic_vector(8 downto 0);
         B : in std_logic_vector(8 downto 0);
         R : out std_logic_vector(8 downto 0));
  end component;

  component pc_plus1 
    port(a : in  std_logic_vector(8 downto 0);
         r : out std_logic_vector(8 downto 0));
  end component;

  component registerfile
    port (ReadReg1  : in  std_logic_vector(4 downto 0);
          ReadReg2  : in  std_logic_vector(4 downto 0);
          WriteReg  : in  std_logic_vector(4 downto 0);
          WriteData : in  std_logic_vector(31 downto 0);
          RegWrite  : in  std_logic;
          clk       : in  std_logic;
          ReadData1 : out std_logic_vector(31 downto 0);
          ReadData2 : out std_logic_vector(31 downto 0));
  end component;

  component alu32
    port(A    : in  std_logic_vector(31 downto 0);
         B    : in  std_logic_vector(31 downto 0);
         op   : in  std_logic_vector(2 downto 0);
         R    : out std_logic_vector(31 downto 0);
         OvFw : out std_logic;
         Zero : out std_logic);
  end component;

  component ALUcontrol
    port (ALUop     : in std_logic_vector(1 downto 0);
          funct     : in std_logic_vector(5 downto 0);
          operation : out std_logic_vector(2 downto 0));
  end component;

  component maincontrol 
    port (Opcode   : in  std_logic_vector(5 downto 0);
          RegDst   : out std_logic;
          ALUSrc   : out std_logic;
          MemtoReg : out std_logic;
          RegWrite : out std_logic;
          MemRead  : out std_logic;
          MemWrite : out std_logic;
          Branch   : out std_logic;
          ALUop    : out std_logic_vector(1 downto 0));
  end component;

  component SignExtend
    port (A : in std_logic_vector(15 downto 0);
          Y : out std_logic_vector(31 downto 0));
  end component;

  signal  aluop : std_logic_vector (1 downto 0);
  signal  regdst, alusrc, memtoreg, regwrite, memread, memwrite, branch   : std_logic;
  signal  writereg : std_logic_vector (4 downto 0);
  signal  writedata, readdata1, readdata2 : std_logic_vector (31 downto 0);
  signal  op : std_logic_vector (2 downto 0);
  signal  B, alu_res : std_logic_vector (31 downto 0);
  signal  clk_bar, alu_ovf, alu_zr, bra_and_zr : std_logic; 
  signal  instr_extended : std_logic_vector (31 downto 0);
  signal  PC_out, pcplus1, pc_adder_res, bra_mux_out, jmp_mux_out : std_logic_vector (8 downto 0);

begin

  PC_reg : register9 port map(bra_mux_out, clk, reset, '1', PC_out);
  RF : registerfile port map(instr(25 downto 21), instr(20 downto 16),
                             writereg, writedata, regwrite, clk_bar, readdata1, readdata2);
  main_ctrl : maincontrol port map(instr (31 downto 26), regdst, alusrc, memtoreg,
                                   regwrite, memread, memwrite, branch, aluop);
  alu_ctrl : ALUcontrol port map(aluop, instr (5 downto 0), op);
  mp_alu : alu32 port map(readdata1, B, op, alu_res, alu_ovf, alu_zr);
  extend : SignExtend port map(instr (15 downto 0), instr_extended);  
  pc_incr : pc_plus1 port map(PC_out, pcplus1);
  pc_adder : rca9 port map(pcplus1, instr_extended(8 downto 0), pc_adder_res);  
  mux_rf : mux21x5 port map(instr(20 downto 16), instr(15 downto 11), regdst, writereg);
  bra_mux : mux21x9 port map(pcplus1, pc_adder_res, bra_and_zr, bra_mux_out);
  alu_amux : mux21x32 port map(readdata2, instr_extended, alusrc, B);
  datamem_mux : mux21x32 port map(alu_res, data_read, memtoreg, writedata);
  
  clk_bar <= not clk;
  ins_addr <= PC_out;
  stb <= memread or memwrite;
  we <= memwrite;
  bra_and_zr <= branch and alu_zr;
  mem_write_data <= readdata2;
  data_addr <= alu_res(8 downto 0);

end struct;
