-- 
-- Author: John Moskal
--  Created: 03-11-2005
-- Modified: 03-31-2005
--
-- Modifications/Comments:
-- Possible Hazards:
-- E Stage data dependent on M or W Stage - forward values from M or W
-- RegWrite to $0 - always use value from RegFile (no forwarding)
-- Branching Hazard - forward from E, M or W
------------------------------------------------------------------------------------------

library work;
library ieee;
use ieee.std_logic_1164.all;

entity FwdUnit is
port
(
   CM_RegWrite,
   CW_RegWrite:         in std_logic;
   CM_MemAluMux,
   CW_MemAluMux:          in std_logic;
   M_DstRegAdr,
   W_DstRegAdr:         in std_logic_vector(4 downto 0);
   D_RsAdr,
   E_RsAdr,
   D_RtAdr,
   E_RtAdr:            in std_logic_vector(4 downto 0);
   C_FwdMux1,
   C_FwdMux2,
   C_CompMux1,
   C_CompMux2:         out std_logic_vector(1 downto 0)
);
end entity FwdUnit;

------------------------------------------------------------------------------------------

architecture stuff of FwdUnit is

-- COMPONENTS --

-- SIGNALS --

-- HDL --

begin

---------------------------------------------------------------
-- Forwarding Logic for FwdMux1
--
-- Forward from MA if:
--    M_DstRegAdr is not $0 (not writable)
--    Instruction @ MA is not LW (CM_MemAluMux is '1')
--    D_Rs is dependent on M_AluRslt (M_DstRegAdr)
-- Forward from WB if:
--    W_DstRegAdr is not $0 (not writable)
--    D_Rs is dependent on W_Data or W_AluRslt (that is on W_Fwd) 


process (CM_RegWrite, CM_MemAluMux, M_DstRegAdr,  CW_RegWrite, W_DstRegAdr, E_RsAdr)
begin

if (not(M_DstRegAdr = "00000") and (CM_RegWrite = '1') and (CM_MemAluMux = '1') and (M_DstRegAdr = E_RsAdr)) then
    C_FwdMux1 <= "01";
elsif (not(W_DstRegAdr = "00000") and (CW_RegWrite = '1') and (W_DstRegAdr = E_RsAdr)) then
    C_FwdMux1 <= "10";
else
    C_FwdMux1 <= "00";
end if;

end process;

---------------------------------------------------------------
-- Forwarding Logic for FwdMux2
--
-- Forward from MA if:
--    M_DstRegAdr is not $0 (not writable)
--    Instruction @ MA is not LW (CM_MemAluMux is '1')
--    D_Rt is dependent on M_AluRslt (M_DstRegAdr)
-- Forward from WB if:
--    W_DstRegAdr is not $0 (not writable)
--    D_Rt is dependent on W_Data or W_AluRslt (that is on W_Fwd) 

process (CM_RegWrite, CM_MemAluMux, M_DstRegAdr,  CW_RegWrite, W_DstRegAdr, E_RtAdr)
begin

if (not(M_DstRegAdr = "00000") and (CM_RegWrite = '1') and (CM_MemAluMux = '1') and (M_DstRegAdr = E_RtAdr)) then
    C_FwdMux2 <= "01";
elsif (not(W_DstRegAdr = "00000") and (CW_RegWrite = '1') and (W_DstRegAdr = E_RtAdr)) then
    C_FwdMux2 <= "10";
else
    C_FwdMux2 <= "00";
end if;

end process;

---------------------------------------------------------------
-- Forwarding Logic for CmpMux1
--
-- Forward from MA if:
--    M_DstRegAdr is not $0 (not writable)
--    Instruction @ MA is not LW (CM_MemAluMux is '1')
--    D_Rs is dependent on M_AluRslt (M_DstRegAdr)
-- Forward from WB if:
--    W_DstRegAdr is not $0 (not writable)
--    D_Rs is dependent on W_Data or W_AluRslt (that is on W_Fwd) 

process(CM_RegWrite, CM_MemAluMux, M_DstRegAdr, CW_RegWrite, W_DstRegAdr, D_RsAdr)
begin

if (not(M_DstRegAdr = "00000") and (CM_RegWrite = '1') and (CM_MemAluMux = '1') and (M_DstRegAdr = D_RsAdr)) then
   C_CompMux1 <= "01";
elsif (not(W_DstRegAdr = "00000") and (CW_RegWrite = '1') and (W_DstRegAdr = D_RsAdr)) then
   C_CompMux1 <= "10";
else
   C_CompMux1 <= "00";
end if;
    
end process;   

---------------------------------------------------------------
-- Forwarding Logic for CmpMux2
--
-- Forward from MA if:
--    M_DstRegAdr is not $0 (not writable)
--    Instruction @ MA is not LW (CM_MemAluMux is '1')
--    D_Rt is dependent on M_AluRslt (M_DstRegAdr)
-- Forward from WB if:
--    W_DstRegAdr is not $0 (not writable)
--    D_Rt is dependent on W_Data or W_AluRslt (that is on W_Fwd) 

process(CM_RegWrite, CM_MemAluMux, M_DstRegAdr, CW_RegWrite, W_DstRegAdr, D_RtAdr)
begin

if (not(M_DstRegAdr = "00000") and (CM_RegWrite = '1') and (CM_MemAluMux = '1') and (M_DstRegAdr = D_RtAdr)) then
   C_CompMux2 <= "01";
elsif (not(W_DstRegAdr = "00000") and (CW_RegWrite = '1') and (W_DstRegAdr = D_RtAdr)) then
   C_CompMux2 <= "10";
else
   C_CompMux2 <= "00";
end if;

end process;


end stuff;

------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------


